use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Sub, SubAssign};

#[derive(Debug, Clone, Copy)]
pub struct PhysVec {
	pub x: f64,
	pub y: f64,
}

impl From<(u32, u32)> for PhysVec {
	fn from(value: (u32, u32)) -> Self {
		PhysVec {
			x: value.0 as f64,
			y: value.1 as f64,
		}
	}
}

impl Add for PhysVec {
	type Output = Self;

	fn add(self, other: Self) -> Self {
		Self {
			x: self.x + other.x,
			y: self.y + other.y,
		}
	}
}

impl AddAssign for PhysVec {
	fn add_assign(&mut self, rhs: Self) {
		self.x += rhs.x;
		self.y += rhs.y;
	}
}

impl Sub for PhysVec {
	type Output = Self;

	fn sub(self, other: Self) -> Self {
		Self {
			x: self.x - other.x,
			y: self.y - other.y,
		}
	}
}

impl SubAssign for PhysVec {
	fn sub_assign(&mut self, rhs: Self) {
		self.x += rhs.x;
		self.y += rhs.y;
	}
}

impl Mul<f64> for PhysVec {
	type Output = PhysVec;
	fn mul(self, rhs: f64) -> Self::Output {
		Self {
			x: self.x * rhs,
			y: self.y * rhs,
		}
	}
}

impl MulAssign<f64> for PhysVec {
	fn mul_assign(&mut self, rhs: f64) {
		self.x *= rhs;
		self.y *= rhs;
	}
}

impl Div<f64> for PhysVec {
	type Output = PhysVec;
	fn div(self, rhs: f64) -> Self::Output {
		Self {
			x: self.x / rhs,
			y: self.y / rhs,
		}
	}
}

impl DivAssign<f64> for PhysVec {
	fn div_assign(&mut self, rhs: f64) {
		self.x /= rhs;
		self.y /= rhs;
	}
}
